@@ -1,21 +1,6 @@
-/*
- * This program is free software; you can redistribute it and/or
- * modify it under the terms of the GNU General Public License
- * as published by the Free Software Foundation; either version 2
- * of the License, or (at your option) any later version.
+/* SPDX-FileCopyrightText: 2001-2002 NaN Holding BV. All rights reserved.
  *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software Foundation,
- * Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
- *
- * The Original Code is Copyright (C) 2001-2002 by NaN Holding BV.
- * All rights reserved.
- */
+ * SPDX-License-Identifier: GPL-2.0-or-later */
 
 /** \file
  * \ingroup DNA
@@ -25,14 +10,15 @@
  * or sequenced in the non-linear-editor (NLA).
  */
 
-#ifndef __DNA_ACTION_TYPES_H__
-#define __DNA_ACTION_TYPES_H__
+#pragma once
 
-#include "DNA_listBase.h"
 #include "DNA_ID.h"
-#include "DNA_view2d_types.h"
-#include "DNA_vec_types.h"
+#include "DNA_armature_types.h"
+#include "DNA_listBase.h"
+#include "DNA_session_uid_types.h"
 #include "DNA_userdef_types.h" /* ThemeWireColor */
+#include "DNA_vec_types.h"
+#include "DNA_view2d_types.h"
 
 struct Collection;
 struct GHash;
@@ -45,7 +31,7 @@ struct SpaceLink;
 /* Motion Paths ------------------------------------ */
 /* (used for Pose Channels and Objects) */
 
-/* Data point for motion path (mpv) */
+/** Data point for motion path (`mpv`). */
 typedef struct bMotionPathVert {
   /** Coordinates of point in 3D-space. */
   float co[3];
@@ -53,7 +39,7 @@ typedef struct bMotionPathVert {
   int flag;
 } bMotionPathVert;
 
-/* bMotionPathVert->flag */
+/** #bMotionPathVert::flag */
 typedef enum eMotionPathVert_Flag {
   /* vert is selected */
   MOTIONPATH_VERT_SEL = (1 << 0),
@@ -71,9 +57,9 @@ typedef struct bMotionPath {
   /** The number of cached verts. */
   int length;
 
-  /** For drawing paths, the start frame number. */
+  /** For drawing paths, the start frame number. Inclusive.*/
   int start_frame;
-  /** For drawing paths, the end frame number. */
+  /** For drawing paths, the end frame number. Exclusive. */
   int end_frame;
 
   /** Optional custom color. */
@@ -100,6 +86,8 @@ typedef enum eMotionPath_Flag {
   MOTIONPATH_FLAG_CUSTOM = (1 << 2),
   /* Draw lines or only points */
   MOTIONPATH_FLAG_LINES = (1 << 3),
+  /* Bake to scene camera. */
+  MOTIONPATH_FLAG_BAKE_CAMERA = (1 << 4),
 } eMotionPath_Flag;
 
 /* Visualization General --------------------------- */
@@ -116,14 +104,16 @@ typedef struct bAnimVizSettings {
   short path_type;
   /** Number of frames between points indicated on the paths. */
   short path_step;
+  /** #eMotionPath_Ranges. */
+  short path_range;
 
   /** #eMotionPaths_ViewFlag. */
   short path_viewflag;
   /** #eMotionPaths_BakeFlag. */
   short path_bakeflag;
-  char _pad[6];
+  char _pad[4];
 
-  /** Start and end frames of path-calculation range. */
+  /** Start and end frames of path-calculation range. Both are inclusive.*/
   int path_sf, path_ef;
   /** Number of frames before/after current frame to show. */
   int path_bc, path_ac;
@@ -131,7 +121,7 @@ typedef struct bAnimVizSettings {
 
 /* bAnimVizSettings->recalc */
 typedef enum eAnimViz_RecalcFlags {
-  /* motionpaths need recalculating */
+  /* Motion-paths need recalculating. */
   ANIMVIZ_RECALC_PATHS = (1 << 0),
 } eAnimViz_RecalcFlags;
 
@@ -143,6 +133,15 @@ typedef enum eMotionPaths_Types {
   MOTIONPATH_TYPE_ACFRA = 1,
 } eMotionPath_Types;
 
+/* bAnimVizSettings->path_range */
+typedef enum eMotionPath_Ranges {
+  /* Default is scene */
+  MOTIONPATH_RANGE_SCENE = 0,
+  MOTIONPATH_RANGE_KEYS_SELECTED = 1,
+  MOTIONPATH_RANGE_KEYS_ALL = 2,
+  MOTIONPATH_RANGE_MANUAL = 3,
+} eMotionPath_Ranges;
+
 /* bAnimVizSettings->path_viewflag */
 typedef enum eMotionPaths_ViewFlag {
   /* show frames on path */
@@ -154,18 +153,20 @@ typedef enum eMotionPaths_ViewFlag {
   /* find keyframes in whole action (instead of just in matching group name) */
   MOTIONPATH_VIEW_KFACT = (1 << 3),
   /* draw lines on path */
-  MOTIONPATH_VIEW_LINES = (1 << 4),
+  /* MOTIONPATH_VIEW_LINES = (1 << 4), */ /* UNUSED */
 } eMotionPath_ViewFlag;
 
 /* bAnimVizSettings->path_bakeflag */
 typedef enum eMotionPaths_BakeFlag {
   /** motion paths directly associated with this block of settings needs updating */
-  MOTIONPATH_BAKE_NEEDS_RECALC = (1 << 0),
+  /* MOTIONPATH_BAKE_NEEDS_RECALC = (1 << 0), */ /* UNUSED */
   /** for bones - calculate head-points for curves instead of tips */
   MOTIONPATH_BAKE_HEADS = (1 << 1),
   /** motion paths exist for AnimVizSettings instance - set when calc for first time,
    * and unset when clearing */
   MOTIONPATH_BAKE_HAS_PATHS = (1 << 2),
+  /* Bake the path in camera space. */
+  MOTIONPATH_BAKE_CAMERA_SPACE = (1 << 3),
 } eMotionPath_BakeFlag;
 
 /* runtime */
@@ -183,13 +184,35 @@ typedef struct bPoseChannelDrawData {
 struct DualQuat;
 struct Mat4;
 
+/* Describes a plane in pose space that delimits B-Bone segments. */
+typedef struct bPoseChannel_BBoneSegmentBoundary {
+  /* Boundary data in pose space. */
+  float point[3];
+  float plane_normal[3];
+  /* Dot product of point and plane_normal to speed up distance computation. */
+  float plane_offset;
+
+  /**
+   * Inverse width of the smoothing at this level in head-tail space.
+   * Optimization: this value is actually indexed by BSP depth (0 to `bsp_depth - 1`), not joint
+   * index. It's put here to avoid allocating a separate array by utilizing the padding space.
+   */
+  float depth_scale;
+} bPoseChannel_BBoneSegmentBoundary;
+
 typedef struct bPoseChannel_Runtime {
+  SessionUID session_uid;
+
   /* Cached dual quaternion for deformation. */
   struct DualQuat deform_dual_quat;
 
   /* B-Bone shape data: copy of the segment count for validation. */
   int bbone_segments;
 
+  /* Inverse of the total length of the segment polyline. */
+  float bbone_arc_length_reciprocal;
+  char _pad1[4];
+
   /* Rest and posed matrices for segments. */
   struct Mat4 *bbone_rest_mats;
   struct Mat4 *bbone_pose_mats;
@@ -197,6 +220,10 @@ typedef struct bPoseChannel_Runtime {
   /* Delta from rest to pose in matrix and DualQuat form. */
   struct Mat4 *bbone_deform_mats;
   struct DualQuat *bbone_dual_quats;
+
+  /* Segment boundaries for curved mode. */
+  struct bPoseChannel_BBoneSegmentBoundary *bbone_segment_boundaries;
+  void *_pad;
 } bPoseChannel_Runtime;
 
 /* ************************************************ */
@@ -204,12 +231,15 @@ typedef struct bPoseChannel_Runtime {
 
 /* PoseChannel ------------------------------------ */
 
-/* PoseChannel
+/**
+ * PoseChannel
  *
- * A PoseChannel stores the results of Actions and transform information
- * with respect to the restposition of Armature bones
+ * A #bPoseChannel stores the results of Actions and transform information
+ * with respect to the rest-position of #bArmature bones.
  */
 typedef struct bPoseChannel {
+  DNA_DEFINE_CXX_METHODS(bPoseChannel)
+
   struct bPoseChannel *next, *prev;
 
   /** User-Defined Properties on this PoseChannel. */
@@ -250,17 +280,24 @@ typedef struct bPoseChannel {
 
   /** Motion path cache for this bone. */
   bMotionPath *mpath;
-  /** Draws custom object instead of default bone shape. */
+  /**
+   * Draws custom object instead of default bone shape.
+   *
+   * \note For the purpose of user interaction (selection, display etc),
+   * it's important this value is treated as NULL when #ARM_NO_CUSTOM is set.
+   */
   struct Object *custom;
   /**
-   * Odd feature, display with another bones transform.
-   * needed in rare cases for advanced rigs,
-   * since the alternative is highly complicated - campbell
+   * This is a specific feature to display with another bones transform.
+   * Needed in rare cases for advanced rigs, since alternative solutions are highly complicated.
+   *
+   * \note This depends #bPoseChannel.custom being set and the #ARM_NO_CUSTOM flag being unset.
    */
   struct bPoseChannel *custom_tx;
-  float custom_scale;
-
-  char _pad1[4];
+  float custom_scale; /* Deprecated */
+  float custom_scale_xyz[3];
+  float custom_translation[3];
+  float custom_rotation_euler[3];
 
   /** Transforms - written in by actions or transform. */
   float loc[3];
@@ -280,20 +317,24 @@ typedef struct bPoseChannel {
   short rotmode;
   char _pad[2];
 
-  /** Matrix result of loc/quat/size, and where we put deform in, see next line */
+  /**
+   * Matrix result of location/rotation/scale components, and evaluation of
+   * animation data and constraints.
+   *
+   * This is the dynamic component of `pose_mat` (without #Bone.arm_mat).
+   */
   float chan_mat[4][4];
   /**
-   * Constraints accumulate here. in the end, pose_mat = bone->arm_mat * chan_mat
-   * this matrix is object space.
+   * Channel matrix in the armature object space, i.e. `pose_mat = bone->arm_mat * chan_mat`.
    */
   float pose_mat[4][4];
   /** For display, pose_mat with bone length applied. */
   float disp_mat[4][4];
-  /** For display, pose_mat with bone length applied and translated to tai.l*/
+  /** For display, pose_mat with bone length applied and translated to tail. */
   float disp_tail_mat[4][4];
   /**
    * Inverse result of constraints.
-   * doesn't include effect of restposition, parent, and local transform.
+   * doesn't include effect of rest-position, parent, and local transform.
    */
   float constinv[4][4];
 
@@ -317,11 +358,12 @@ typedef struct bPoseChannel {
    * and are applied on top of the copies in pchan->bone
    */
   float roll1, roll2;
-  float curve_in_x, curve_in_y;
-  float curve_out_x, curve_out_y;
+  float curve_in_x, curve_in_z;
+  float curve_out_x, curve_out_z;
   float ease1, ease2;
-  float scale_in_x, scale_in_y;
-  float scale_out_x, scale_out_y;
+  float scale_in_x DNA_DEPRECATED, scale_in_z DNA_DEPRECATED;
+  float scale_out_x DNA_DEPRECATED, scale_out_z DNA_DEPRECATED;
+  float scale_in[3], scale_out[3];
 
   /** B-Bone custom handles; set on read file or rebuild pose based on pchan->bone data. */
   struct bPoseChannel *bbone_prev;
@@ -335,6 +377,8 @@ typedef struct bPoseChannel {
   /** Points to an original pose channel. */
   struct bPoseChannel *orig_pchan;
 
+  BoneColor color; /* MUST be named the same as in Bone and EditBone structs. */
+
   /** Runtime data (keep last). */
   struct bPoseChannel_Runtime runtime;
 } bPoseChannel;
@@ -360,7 +404,7 @@ typedef enum ePchan_Flag {
   POSE_DONE = (1 << 10),
   /* visualization */
   POSE_KEY = (1 << 11),
-  POSE_STRIDE = (1 << 12),
+  /* POSE_STRIDE = (1 << 12), */ /* UNUSED */
   /* standard IK solving */
   POSE_IKTREE = (1 << 13),
 #if 0
@@ -373,16 +417,15 @@ typedef enum ePchan_Flag {
 
 /* PoseChannel constflag (constraint detection) */
 typedef enum ePchan_ConstFlag {
-  PCHAN_HAS_IK = (1 << 0),
-  PCHAN_HAS_CONST = (1 << 1),
-  /* only used for drawing Posemode, not stored in channel */
-  PCHAN_HAS_ACTION = (1 << 2),
-  PCHAN_HAS_TARGET = (1 << 3),
-  /* only for drawing Posemode too */
-  PCHAN_HAS_STRIDE = (1 << 4),
-  /* spline IK */
-  PCHAN_HAS_SPLINEIK = (1 << 5),
+  PCHAN_HAS_IK = (1 << 0),           /* Has IK constraint. */
+  PCHAN_HAS_CONST = (1 << 1),        /* Has any constraint. */
+  /* PCHAN_HAS_ACTION = (1 << 2), */ /* UNUSED */
+  PCHAN_HAS_NO_TARGET = (1 << 3),    /* Has (spline) IK constraint but no target is set. */
+  /* PCHAN_HAS_STRIDE = (1 << 4), */ /* UNUSED */
+  PCHAN_HAS_SPLINEIK = (1 << 5),     /* Has Spline IK constraint. */
+  PCHAN_INFLUENCED_BY_IK = (1 << 6), /* Is part of a (non-spline) IK chain. */
 } ePchan_ConstFlag;
+ENUM_OPERATORS(ePchan_ConstFlag, PCHAN_INFLUENCED_BY_IK);
 
 /* PoseChannel->ikflag */
 typedef enum ePchan_IkFlag {
@@ -407,9 +450,9 @@ typedef enum ePchan_DrawFlag {
   PCHAN_DRAW_NO_CUSTOM_BONE_SIZE = (1 << 0),
 } ePchan_DrawFlag;
 
-#define PCHAN_CUSTOM_DRAW_SIZE(pchan) \
-  (pchan)->custom_scale *( \
-      ((pchan)->drawflag & PCHAN_DRAW_NO_CUSTOM_BONE_SIZE) ? 1.0f : (pchan)->bone->length)
+/* NOTE: It doesn't take custom_scale_xyz into account. */
+#define PCHAN_CUSTOM_BONE_LENGTH(pchan) \
+  (((pchan)->drawflag & PCHAN_DRAW_NO_CUSTOM_BONE_SIZE) ? 1.0f : (pchan)->bone->length)
 
 #ifdef DNA_DEPRECATED_ALLOW
 /* PoseChannel->bboneflag */
@@ -427,7 +470,7 @@ typedef enum ePchan_BBoneFlag {
 typedef enum eRotationModes {
   /* quaternion rotations (default, and for older Blender versions) */
   ROT_MODE_QUAT = 0,
-  /* euler rotations - keep in sync with enum in BLI_math.h */
+  /* euler rotations - keep in sync with enum in BLI_math_rotation.h */
   /** Blender 'default' (classic) - must be as 1 to sync with BLI_math_rotation.h defines */
   ROT_MODE_EUL = 1,
   ROT_MODE_XYZ = 1,
@@ -456,7 +499,7 @@ typedef enum eRotationModes {
 typedef struct bPose {
   /** List of pose channels, PoseBones in RNA. */
   ListBase chanbase;
-  /** Ghash for quicker string lookups. */
+  /** Use a hash-table for quicker string lookups. */
   struct GHash *chanhash;
 
   /* Flat array of pose channels. It references pointers from
@@ -466,9 +509,6 @@ typedef struct bPose {
 
   short flag;
   char _pad[2];
-  /** Proxy layer: copy from armature, gets synced. */
-  unsigned int proxy_layer;
-  char _pad1[4];
 
   /** Local action time of this pose. */
   float ctime;
@@ -491,8 +531,6 @@ typedef struct bPose {
 
   /** Settings for visualization of bone animation. */
   bAnimVizSettings avs;
-  /** Proxy active bone name, MAXBONENAME. */
-  char proxy_act_bone[64];
 } bPose;
 
 /* Pose->flag */
@@ -506,7 +544,7 @@ typedef enum ePose_Flags {
   /* pose has constraints which depend on time (used when depsgraph updates for a new frame) */
   POSE_CONSTRAINTS_TIMEDEPEND = (1 << 3),
   /* recalculate bone paths */
-  POSE_RECALCPATHS = (1 << 4),
+  /* POSE_RECALCPATHS = (1 << 4), */ /* UNUSED */
   /* set by BKE_pose_rebuild to give a chance to the IK solver to rebuild IK tree */
   POSE_WAS_REBUILT = (1 << 5),
   POSE_FLAG_DEPRECATED = (1 << 6), /* deprecated. */
@@ -558,6 +596,11 @@ typedef enum eItasc_Flags {
   ITASC_INITIAL_REITERATION = (1 << 1),
   ITASC_REITERATION = (1 << 2),
   ITASC_SIMULATION = (1 << 3),
+  /**
+   * Set this flag to always translate root bones (i.e. bones without a parent) to (0, 0, 0).
+   * This was the pre-3.6 behavior, and this flag was introduced for backward compatibility.
+   */
+  ITASC_TRANSLATE_ROOT_BONES = (1 << 4),
 } eItasc_Flags;
 
 /* bItasc->solver */
@@ -590,7 +633,7 @@ typedef struct bActionGroup {
   struct bActionGroup *next, *prev;
 
   /**
-   * Note: this must not be touched by standard listbase functions
+   * NOTE: this must not be touched by standard listbase functions
    * which would clear links to other channels.
    */
   ListBase channels;
@@ -635,18 +678,19 @@ typedef enum eActionGroup_Flag {
 
 /* Actions -------------------------------------- */
 
-/* Action - reusable F-Curve 'bag'  (act)
+/**
+ * Action - reusable F-Curve 'bag'  (act)
  *
- * This contains F-Curves that may affect settings from more than one ID blocktype and/or datablock
- * (i.e. sub-data linked/used directly to the ID block that the animation data is linked to),
- * but with the restriction that the other unrelated data (i.e. data that is not directly used or
- * linked to by the source ID block).
+ * This contains F-Curves that may affect settings from more than one ID block-type and/or
+ * data-block (i.e. sub-data linked/used directly to the ID block that the animation data is linked
+ * to), but with the restriction that the other unrelated data (i.e. data that is not directly used
+ * or linked to by the source ID block).
  *
  * It serves as a 'unit' of reusable animation information (i.e. keyframes/motion data),
  * that affects a group of related settings (as defined by the user).
  */
 typedef struct bAction {
-  /** ID-serialisation for relinking. */
+  /** ID-serialization for relinking. */
   ID id;
 
   /** Function-curves (FCurve). */
@@ -669,6 +713,14 @@ typedef struct bAction {
    */
   int idroot;
   char _pad[4];
+
+  /**
+   * Start and end of the manually set intended playback frame range. Used by UI and
+   * some editing tools, but doesn't directly affect animation evaluation in any way.
+   */
+  float frame_start, frame_end;
+
+  PreviewImage *preview;
 } bAction;
 
 /* Flags for the action */
@@ -679,8 +731,12 @@ typedef enum eAction_Flags {
 
   /* flags for evaluation/editing */
   ACT_MUTED = (1 << 9),
-  ACT_PROTECTED = (1 << 10),
-  ACT_DISABLED = (1 << 11),
+  /* ACT_PROTECTED = (1 << 10), */ /* UNUSED */
+  /* ACT_DISABLED = (1 << 11), */  /* UNUSED */
+  /** The action has a manually set intended playback frame range. */
+  ACT_FRAME_RANGE = (1 << 12),
+  /** The action is intended to be a cycle (requires ACT_FRAME_RANGE). */
+  ACT_CYCLIC = (1 << 13),
 } eAction_Flags;
 
 /* ************************************************ */
@@ -690,7 +746,7 @@ typedef enum eAction_Flags {
 typedef struct bDopeSheet {
   /** Currently ID_SCE (for Dopesheet), and ID_SC (for Grease Pencil). */
   ID *source;
-  /** Cache for channels (only initialized when pinned). */  // XXX not used!
+  /** Cache for channels (only initialized when pinned). */ /* XXX not used! */
   ListBase chanbase;
 
   /** Object group for option to only include objects that belong to this Collection. */
@@ -698,8 +754,9 @@ typedef struct bDopeSheet {
   /** String to search for in displayed names of F-Curves, or NlaTracks/GP Layers/etc. */
   char searchstr[64];
 
-  /** Flags to use for filtering data. */
+  /** Flags to use for filtering data #eAnimFilter_Flags. */
   int filterflag;
+  /** #eDopeSheet_FilterFlag2 */
   int filterflag2;
   /** Standard flags. */
   int flag;
@@ -759,20 +816,24 @@ typedef enum eDopeSheet_FilterFlag {
   /** show only F-Curves which are disabled/have errors - for debugging drivers */
   ADS_FILTER_ONLY_ERRORS = (1 << 28),
 
-  /* GPencil Mode */
-  /** GP Mode - Only show datablocks used in the scene */
-  ADS_FILTER_GP_3DONLY = (1 << 29),
-
+#if 0
   /** combination filters (some only used at runtime) */
   ADS_FILTER_NOOBDATA = (ADS_FILTER_NOCAM | ADS_FILTER_NOMAT | ADS_FILTER_NOLAM |
                          ADS_FILTER_NOCUR | ADS_FILTER_NOPART | ADS_FILTER_NOARM |
                          ADS_FILTER_NOSPK | ADS_FILTER_NOMODIFIERS),
+#endif
 } eDopeSheet_FilterFlag;
 
 /* DopeSheet filter-flags - Overflow (filterflag2) */
 typedef enum eDopeSheet_FilterFlag2 {
   ADS_FILTER_NOCACHEFILES = (1 << 1),
   ADS_FILTER_NOMOVIECLIPS = (1 << 2),
+  ADS_FILTER_NOHAIR = (1 << 3),
+  ADS_FILTER_NOPOINTCLOUD = (1 << 4),
+  ADS_FILTER_NOVOLUME = (1 << 5),
+
+  /** Include working drivers with variables using their fallback values into Only Show Errors. */
+  ADS_FILTER_DRIVER_FALLBACK_AS_ERROR = (1 << 6),
 } eDopeSheet_FilterFlag2;
 
 /* DopeSheet general flags */
@@ -787,6 +848,8 @@ typedef enum eDopeSheet_Flag {
   ADS_FLAG_FUZZY_NAMES = (1 << 2),
   /** do not sort datablocks (mostly objects) by name (NOTE: potentially expensive operation) */
   ADS_FLAG_NO_DB_SORT = (1 << 3),
+  /** Invert the search filter */
+  ADS_FLAG_INVERT_FILTER = (1 << 4),
 } eDopeSheet_Flag;
 
 typedef struct SpaceAction_Runtime {
@@ -812,7 +875,7 @@ typedef struct SpaceAction {
   /** The currently active context (when not showing action). */
   bDopeSheet ads;
 
-  /** For Time-Slide transform mode drawing - current frame?. */
+  /** For Time-Slide transform mode drawing - current frame? */
   float timeslide;
 
   short flag;
@@ -820,8 +883,8 @@ typedef struct SpaceAction {
   char mode;
   /* Storage for sub-space types. */
   char mode_prev;
-  /** Automatic keyframe snapping mode  . */
-  char autosnap;
+  /* Snapping now lives on the Scene. */
+  char autosnap DNA_DEPRECATED;
   /** (eTimeline_Cache_Flag). */
   char cache_display;
   char _pad1[6];
@@ -838,17 +901,16 @@ typedef enum eSAction_Flag {
   /* draw time in seconds instead of time in frames */
   SACTION_DRAWTIME = (1 << 2),
   /* don't filter action channels according to visibility */
-  // SACTION_NOHIDE = (1 << 3), // XXX deprecated... old animation system
+  // SACTION_NOHIDE = (1 << 3), /* Deprecated, old animation systems. */
   /* don't kill overlapping keyframes after transform */
   SACTION_NOTRANSKEYCULL = (1 << 4),
   /* don't include keyframes that are out of view */
-  // SACTION_HORIZOPTIMISEON = (1 << 5), // XXX deprecated... old irrelevant trick
-  /* show pose-markers (local to action) in Action Editor mode  */
+  // SACTION_HORIZOPTIMISEON = (1 << 5), /* Deprecated, old irrelevant trick. */
+  /* show pose-markers (local to action) in Action Editor mode. */
   SACTION_POSEMARKERS_SHOW = (1 << 6),
   /* don't draw action channels using group colors (where applicable) */
-  SACTION_NODRAWGCOLORS = (1 << 7),
-  /* don't draw current frame number beside frame indicator */
-  SACTION_NODRAWCFRANUM = (1 << 8),
+  /* SACTION_NODRAWGCOLORS = (1 << 7), DEPRECATED */
+  /* SACTION_NODRAWCFRANUM = (1 << 8), DEPRECATED */
   /* don't perform realtime updates */
   SACTION_NOREALTIMEUPDATES = (1 << 10),
   /* move markers as well as keyframes */
@@ -857,38 +919,36 @@ typedef enum eSAction_Flag {
   SACTION_SHOW_INTERPOLATION = (1 << 12),
   /* show extremes */
   SACTION_SHOW_EXTREMES = (1 << 13),
-  /* show vertical line markers */
-  SACTION_SHOW_MARKER_LINES = (1 << 14),
+  /* show markers region */
+  SACTION_SHOW_MARKERS = (1 << 14),
 } eSAction_Flag;
 
-/* SpaceAction_Runtime.flag */
+/** #SpaceAction_Runtime.flag */
 typedef enum eSAction_Runtime_Flag {
   /** Temporary flag to force channel selections to be synced with main */
   SACTION_RUNTIME_FLAG_NEED_CHAN_SYNC = (1 << 0),
 } eSAction_Runtime_Flag;
 
-/* SpaceAction Mode Settings */
+/** #SpaceAction.mode */
 typedef enum eAnimEdit_Context {
-  /* action on the active object */
+  /** Action on the active object. */
   SACTCONT_ACTION = 0,
-  /* list of all shapekeys on the active object, linked with their F-Curves */
+  /** List of all shape-keys on the active object, linked with their F-Curves. */
   SACTCONT_SHAPEKEY = 1,
-  /* editing of gpencil data */
+  /** Editing of grease-pencil data. */
   SACTCONT_GPENCIL = 2,
-  /* dopesheet (default) */
+  /** Dope-sheet (default). */
   SACTCONT_DOPESHEET = 3,
-  /* mask */
+  /** Mask. */
   SACTCONT_MASK = 4,
-  /* cache file */
+  /** Cache file */
   SACTCONT_CACHEFILE = 5,
-  /* timeline - replacement for the standalone "timeline editor" */
+  /** Timeline - replacement for the standalone "timeline editor". */
   SACTCONT_TIMELINE = 6,
 } eAnimEdit_Context;
 
-/* SpaceAction AutoSnap Settings (also used by other Animation Editors) */
+/* Old snapping enum that is only needed because of the versioning code. */
 typedef enum eAnimEdit_AutoSnap {
-  /* no auto-snap */
-  SACTSNAP_OFF = 0,
   /* snap to 1.0 frame/second intervals */
   SACTSNAP_STEP = 1,
   /* snap to actual frames/seconds (nla-action time) */
@@ -899,7 +959,7 @@ typedef enum eAnimEdit_AutoSnap {
   SACTSNAP_SECOND = 4,
   /* snap to 1.0 second increments */
   SACTSNAP_TSTEP = 5,
-} eAnimEdit_AutoSnap;
+} eAnimEdit_AutoSnap DNA_DEPRECATED;
 
 /* SAction->cache_display */
 typedef enum eTimeline_Cache_Flag {
@@ -910,6 +970,7 @@ typedef enum eTimeline_Cache_Flag {
   TIME_CACHE_SMOKE = (1 << 4),
   TIME_CACHE_DYNAMICPAINT = (1 << 5),
   TIME_CACHE_RIGIDBODY = (1 << 6),
+  TIME_CACHE_SIMULATION_NODES = (1 << 7),
 } eTimeline_Cache_Flag;
 
 /* ************************************************ */
@@ -943,17 +1004,3 @@ typedef struct bActionChannel {
   /** Temporary setting - may be used to indicate group that channel belongs to during syncing. */
   int temp;
 } bActionChannel;
-
-/* Action Channel flags (ONLY USED FOR DO_VERSIONS...) */
-typedef enum eActionChannelFlag {
-  ACHAN_SELECTED = (1 << 0),
-  ACHAN_HIGHLIGHTED = (1 << 1),
-  ACHAN_HIDDEN = (1 << 2),
-  ACHAN_PROTECTED = (1 << 3),
-  ACHAN_EXPANDED = (1 << 4),
-  ACHAN_SHOWIPO = (1 << 5),
-  ACHAN_SHOWCONS = (1 << 6),
-  ACHAN_MOVED = (1u << 31),
-} eActionChannelFlag;
-
-#endif /* __DNA_ACTION_TYPES_H__ */
